@@ -1,6 +1,6 @@
 /*
  *
- * Copyright (C) 2019-2021 Intel Corporation
+ * Copyright (C) 2019-2025 Intel Corporation
  *
  * SPDX-License-Identifier: MIT
  *
@@ -18,7 +18,7 @@ namespace ze_lib
 {
     ///////////////////////////////////////////////////////////////////////////////
     context_t *context = nullptr;
-    #ifdef DYNAMIC_LOAD_LOADER
+    #ifdef L0_STATIC_LOADER_BUILD
     void context_at_exit_destructor()
     {
         if (ze_lib::context) {
@@ -27,7 +27,37 @@ namespace ze_lib
         }
     }
     bool delayContextDestruction = false;
+    bool loaderTeardownCallbackReceived = false;
+    bool loaderTeardownRegistrationEnabled = false;
+
+    /// @brief Callback function to handle loader teardown events.
+    ///
+    /// This function sets the `loaderTeardownCallbackReceived` flag to true,
+    /// indicating that a loader teardown callback has been received.
+    /// It is intended to be used as a static callback during the loader's
+    /// teardown process.
+    void staticLoaderTeardownCallback() {
+        loaderTeardownCallbackReceived = true;
+    }
     #endif
+    /**
+     * @brief Removes a teardown callback from the context's callback registry.
+     *
+     * This function checks if a teardown callback with the specified index exists
+     * in the context's teardownCallbacks map. If it exists, the callback is removed.
+     *
+     * @param index The unique identifier of the teardown callback to remove.
+     */
+    void applicationTeardownCallback(uint32_t index) {
+        std::lock_guard<std::mutex> lock(ze_lib::context->teardownCallbacksMutex);
+        if (ze_lib::context->teardownCallbacks.find(index) != ze_lib::context->teardownCallbacks.end()) {
+            if (ze_lib::context->debugTraceEnabled) {
+                std::string message = "applicationTeardownCallback received for index: " + std::to_string(index);
+                ze_lib::context->debug_trace_message(message, "");
+            }
+            ze_lib::context->teardownCallbacks.erase(index);
+        }
+    }
     bool destruction = false;
 
     ///////////////////////////////////////////////////////////////////////////////
@@ -39,10 +69,20 @@ namespace ze_lib
     ///////////////////////////////////////////////////////////////////////////////
     __zedlllocal context_t::~context_t()
     {
-#ifdef DYNAMIC_LOAD_LOADER
+#ifdef L0_STATIC_LOADER_BUILD
+        if (loaderTeardownRegistrationEnabled && !loaderTeardownCallbackReceived) {
+            loaderTeardownCallback(loaderTeardownCallbackIndex);
+        }
         if (loader) {
             FREE_DRIVER_LIBRARY( loader );
         }
+#else
+        // Given the loader teardown, notify the registered callbacks that the loader is being torn down.
+        for (auto &callback : teardownCallbacks) {
+            callback.second();
+        }
+        // Clear the teardown callbacks map once the callbacks have been executed.
+        teardownCallbacks.clear();
 #endif
         ze_lib::destruction = true;
     };
@@ -52,7 +92,7 @@ namespace ze_lib
     {
         ze_result_t result;
         ze_api_version_t version = ZE_API_VERSION_CURRENT;
-#ifdef DYNAMIC_LOAD_LOADER
+#ifdef L0_STATIC_LOADER_BUILD
         std::string loaderLibraryPath;
         auto loaderLibraryPathEnv = getenv_string("ZEL_LIBRARY_PATH");
         if (!loaderLibraryPathEnv.empty()) {
@@ -100,7 +140,7 @@ namespace ze_lib
             return result;
         }
 
-        bool zeInitDriversSupport = true;
+        ze_api_version_t current_api_version = version;
         const std::string loader_name = "loader";
         for (auto &component : versions) {
             if (loader_name == component.component_name) {
@@ -111,7 +151,6 @@ namespace ze_lib
                     if (component.component_lib_version.minor < 18) {
                         std::string message = "ze_lib Context Init() Version Does not support zeInitDrivers";
                         debug_trace_message(message, "");
-                        zeInitDriversSupport = false;
                     }
                 } else {
                     std::string message = "ze_lib Context Init() Loader version is too new, returning ";
@@ -121,6 +160,12 @@ namespace ze_lib
             }
         }
 
+        if (version > current_api_version) {
+            version = current_api_version;
+            std::string message = "ze_lib Context Init() Static Loader Requesting Loader API Version v" + std::to_string(ZE_MAJOR_VERSION(version)) + "." + std::to_string(ZE_MINOR_VERSION(version));
+            debug_trace_message(message, "");
+        }
+
         typedef HMODULE (ZE_APICALL *getTracing_t)();
         auto getTracing = reinterpret_cast<getTracing_t>(
             GET_FUNCTION_PTR(loader, "zeLoaderGetTracingHandle") );
@@ -130,14 +175,21 @@ namespace ze_lib
             return ZE_RESULT_ERROR_UNINITIALIZED;
         }
         tracing_lib = getTracing();
-        typedef ze_result_t (ZE_APICALL *zelLoaderTracingLayerInit_t)(std::atomic<ze_dditable_t *> &zeDdiTable);
-        auto loaderTracingLayerInit = reinterpret_cast<zelLoaderTracingLayerInit_t>(
+        typedef ze_result_t (ZE_APICALL *zelLoaderZeTracingLayerInit_t)(std::atomic<ze_dditable_t *> &zeDdiTable);
+        auto loaderZeTracingLayerInit = reinterpret_cast<zelLoaderZeTracingLayerInit_t>(
                 GET_FUNCTION_PTR(loader, "zelLoaderTracingLayerInit") );
-        if (loaderTracingLayerInit == nullptr) {
+        if (loaderZeTracingLayerInit == nullptr) {
             std::string message = "ze_lib Context Init() zelLoaderTracingLayerInit missing, disabling dynamic tracer support ";
             debug_trace_message(message, "");
             this->dynamicTracingSupported = false;
         }
+        typedef ze_result_t (ZE_APICALL *zelLoaderZerTracingLayerInit_t)(std::atomic<zer_dditable_t *> &zerDdiTable);
+        auto loaderZerTracingLayerInit = reinterpret_cast<zelLoaderZerTracingLayerInit_t>(
+                GET_FUNCTION_PTR(loader, "zelLoaderZerTracingLayerInit") );
+        if (loaderZerTracingLayerInit == nullptr) {
+            std::string message = "ze_lib Context Init() zelLoaderZerTracingLayerInit missing, dynamic tracing support for Zer API's will be unavailable ";
+            debug_trace_message(message, "");
+        }
         typedef loader::context_t * (ZE_APICALL *zelLoaderGetContext_t)();
         auto loaderGetContext = reinterpret_cast<zelLoaderGetContext_t>(
                 GET_FUNCTION_PTR(loader, "zelLoaderGetContext") );
@@ -159,24 +211,25 @@ namespace ze_lib
 
         if ( ZE_RESULT_SUCCESS == result )
         {
+            memset(&initialzeDdiTable, 0, sizeof(ze_dditable_t));
+            memset(&initialzetDdiTable, 0, sizeof(zet_dditable_t));
+            memset(&initialzesDdiTable, 0, sizeof(zes_dditable_t));
+            memset(&initialzerDdiTable, 0, sizeof(zer_dditable_t));
             ze_lib::context->zeDdiTable.exchange(&ze_lib::context->initialzeDdiTable);
             ze_lib::context->zetDdiTable.exchange(&ze_lib::context->initialzetDdiTable);
             ze_lib::context->zesDdiTable.exchange(&ze_lib::context->initialzesDdiTable);
+            ze_lib::context->zerDdiTable.exchange(&ze_lib::context->initialzerDdiTable);
         }
 
         // Given zesInit, then zesDrivers needs to be used as the sysmanInstanceDrivers;
         bool loaderContextAccessAllowed = true;
-#ifdef DYNAMIC_LOAD_LOADER
+#ifdef L0_STATIC_LOADER_BUILD
+        loaderContextAccessAllowed = false;
         loader::context_t *loaderContext = nullptr;
-        if (loaderGetContext == nullptr) {
-            loaderContextAccessAllowed = false;
-        } else {
-            loaderContext = loaderGetContext();
-        }
 #else
         loader::context_t *loaderContext = loader::context;
 #endif
-        if (sysmanOnly && loaderContextAccessAllowed) {
+        if (sysmanOnly && loaderContextAccessAllowed && loaderContext != nullptr) {
             loaderContext->sysmanInstanceDrivers = &loaderContext->zesDrivers;
         }
 
@@ -209,6 +262,16 @@ namespace ze_lib
                 debug_trace_message(message, to_string(result));
             }
         }
+        // Init the ZER DDI Tables
+        if (ZE_RESULT_SUCCESS == result)
+        {
+            result = zerDdiTableInit(version);
+            if (result != ZE_RESULT_SUCCESS)
+            {
+                std::string message = "ze_lib Context Init() zerDdiTableInit failed with ";
+                debug_trace_message(message, to_string(result));
+            }
+        }
         // Init the Tracing API DDI Tables
         if( ZE_RESULT_SUCCESS == result )
         {
@@ -221,84 +284,25 @@ namespace ze_lib
         // Init the stored ddi tables for the tracing layer
         if( ZE_RESULT_SUCCESS == result )
         {
-            #ifdef DYNAMIC_LOAD_LOADER
-            if (loaderTracingLayerInit) {
-                result = loaderTracingLayerInit(this->pTracingZeDdiTable);
+            #ifdef L0_STATIC_LOADER_BUILD
+            if (loaderZeTracingLayerInit) {
+                result = loaderZeTracingLayerInit(this->pTracingZeDdiTable);
+                if (result == ZE_RESULT_SUCCESS && loaderZerTracingLayerInit) {
+                    result = loaderZerTracingLayerInit(this->pTracingZerDdiTable);
+                }
             }
             #else
             result = zelLoaderTracingLayerInit(this->pTracingZeDdiTable);
+            if (result == ZE_RESULT_SUCCESS) {
+                result = zelLoaderZerTracingLayerInit(this->pTracingZerDdiTable);
+            }
             #endif
         }
         // End DDI Table Inits
 
-        // Check which drivers and layers can be init on this system.
-        if( ZE_RESULT_SUCCESS == result)
-        {
-            // Check which drivers support the ze_driver_flag_t specified
-            // No need to check if only initializing sysman
-            bool requireDdiReinit = false;
-            #ifdef DYNAMIC_LOAD_LOADER
-            if (zeInitDriversSupport) {
-                typedef ze_result_t (ZE_APICALL *zelLoaderDriverCheck_t)(ze_init_flags_t flags, ze_init_driver_type_desc_t* desc, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool *requireDdiReinit, bool sysmanOnly);
-                auto loaderDriverCheck = reinterpret_cast<zelLoaderDriverCheck_t>(
-                        GET_FUNCTION_PTR(loader, "zelLoaderDriverCheck") );
-                if (loaderDriverCheck == nullptr) {
-                    std::string message = "ze_lib Context Init() zelLoaderDriverCheck missing, returning ";
-                    debug_trace_message(message, to_string(ZE_RESULT_ERROR_UNINITIALIZED));
-                    return ZE_RESULT_ERROR_UNINITIALIZED;
-                }
-                result = loaderDriverCheck(flags, desc, &ze_lib::context->initialzeDdiTable.Global, &ze_lib::context->initialzesDdiTable.Global, &requireDdiReinit, sysmanOnly);
-            } else {
-                typedef ze_result_t (ZE_APICALL *zelLoaderDriverCheck_t)(ze_init_flags_t flags, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool *requireDdiReinit, bool sysmanOnly);
-                auto loaderDriverCheck = reinterpret_cast<zelLoaderDriverCheck_t>(
-                        GET_FUNCTION_PTR(loader, "zelLoaderDriverCheck") );
-                if (loaderDriverCheck == nullptr) {
-                    std::string message = "ze_lib Context Init() zelLoaderDriverCheck missing, returning ";
-                    debug_trace_message(message, to_string(ZE_RESULT_ERROR_UNINITIALIZED));
-                    return ZE_RESULT_ERROR_UNINITIALIZED;
-                }
-                result = loaderDriverCheck(flags, &ze_lib::context->initialzeDdiTable.Global, &ze_lib::context->initialzesDdiTable.Global, &requireDdiReinit, sysmanOnly);
-            }
-            #else
-            result = zelLoaderDriverCheck(flags, desc, &ze_lib::context->initialzeDdiTable.Global, &ze_lib::context->initialzesDdiTable.Global, &requireDdiReinit, sysmanOnly);
-            #endif
-            if (result != ZE_RESULT_SUCCESS) {
-                std::string message = "ze_lib Context Init() zelLoaderDriverCheck failed with ";
-                debug_trace_message(message, to_string(result));
-            }
-            // If a driver was removed from the driver list, then the ddi tables need to be reinit to allow for passthru directly to the driver.
-            if (requireDdiReinit && loaderContextAccessAllowed) {
-                // If a user has already called the core apis, then ddi table reinit is not possible due to handles already being read by the user.
-                if (!sysmanOnly && !ze_lib::context->zeInuse) {
-                    // reInit the ZE DDI Tables
-                    if( ZE_RESULT_SUCCESS == result )
-                    {
-                        result = zeDdiTableInit(version);
-                    }
-                    // reInit the ZET DDI Tables
-                    if( ZE_RESULT_SUCCESS == result )
-                    {
-                        result = zetDdiTableInit(version);
-                    }
-                    // If ze/zet ddi tables have been reinit and no longer use the intercept layer, then handles passed to zelLoaderTranslateHandleInternal do not require translation.
-                    // Setting intercept_enabled==false changes the behavior of zelLoaderTranslateHandleInternal to avoid translation.
-                    // Translation is only required if the intercept layer is enabled for the ZE handle types.
-                    loaderContext->intercept_enabled = false;
-                }
-                // If a user has already called the zes/ze apis, then ddi table reinit is not possible due to handles already being read by the user.
-                if (!(ze_lib::context->zesInuse || ze_lib::context->zeInuse)) {
-                    // reInit the ZES DDI Tables
-                    if( ZE_RESULT_SUCCESS == result )
-                    {
-                        result = zesDdiTableInit(version);
-                    }
-                }
-            }
-        }
-
         if( ZE_RESULT_SUCCESS == result )
         {
-#ifdef DYNAMIC_LOAD_LOADER
+#ifdef L0_STATIC_LOADER_BUILD
             // Init Dynamic Loader's Lib Context:
             auto initDriversLoader = reinterpret_cast<ze_pfnInitDrivers_t>(
                 GET_FUNCTION_PTR(loader, "zeInitDrivers") );
@@ -335,10 +339,30 @@ namespace ze_lib
 #endif
             isInitialized = true;
         }
-        #ifdef DYNAMIC_LOAD_LOADER
-        if (!delayContextDestruction) {
-            std::atexit(context_at_exit_destructor);
-        }
+        #ifdef L0_STATIC_LOADER_BUILD
+        std::call_once(ze_lib::context->initTeardownCallbacksOnce, [this]() {
+            if (!delayContextDestruction) {
+                std::atexit(context_at_exit_destructor);
+            }
+            // Get the function pointer for zelRegisterTeardownCallback from the dynamic loader
+            typedef ze_result_t (ZE_APICALL *zelRegisterTeardownCallback_t)(
+                zel_loader_teardown_callback_t,
+                zel_application_teardown_callback_t*,
+                uint32_t*);
+            auto pfnZelRegisterTeardownCallback = reinterpret_cast<zelRegisterTeardownCallback_t>(
+                GET_FUNCTION_PTR(loader, "zelRegisterTeardownCallback"));
+            if (pfnZelRegisterTeardownCallback != nullptr) {
+                auto register_teardown_result = pfnZelRegisterTeardownCallback(staticLoaderTeardownCallback, &loaderTeardownCallback, &loaderTeardownCallbackIndex);
+                if (register_teardown_result != ZE_RESULT_SUCCESS) {
+                    std::string message = "ze_lib Context Init() zelRegisterTeardownCallback failed with ";
+                    debug_trace_message(message, to_string(register_teardown_result));
+                } else {
+                    loaderTeardownRegistrationEnabled = true;
+                    std::string message = "ze_lib Context Init() zelRegisterTeardownCallback completed for the static loader with ";
+                    debug_trace_message(message, to_string(register_teardown_result));
+                }
+            }
+        });
         #endif
         return result;
     }
@@ -353,7 +377,7 @@ zelLoaderGetVersions(
    size_t *num_elems,                     //Pointer to num versions to get.
    zel_component_version_t *versions)     //Pointer to array of versions. If set to NULL, num_elems is returned
 {
-#ifdef DYNAMIC_LOAD_LOADER
+#ifdef L0_STATIC_LOADER_BUILD
     if(nullptr == ze_lib::context->loader)
         return ZE_RESULT_ERROR_UNINITIALIZED;
     typedef ze_result_t (ZE_APICALL *zelLoaderGetVersions_t)(size_t *num_elems, zel_component_version_t *versions);
@@ -373,7 +397,7 @@ zelLoaderTranslateHandle(
    void **handleOut)
 
 {
-#ifdef DYNAMIC_LOAD_LOADER
+#ifdef L0_STATIC_LOADER_BUILD
     if(nullptr == ze_lib::context->loader)
         return ZE_RESULT_ERROR_UNINITIALIZED;
     typedef ze_result_t (ZE_APICALL *zelLoaderTranslateHandleInternal_t)(zel_handle_type_t handleType, void *handleIn, void **handleOut);
@@ -390,6 +414,15 @@ zelSetDriverTeardown()
 {
     ze_result_t result = ZE_RESULT_SUCCESS;
     if (!ze_lib::destruction) {
+        if (ze_lib::context) {
+            // Given the driver teardown, notify the registered callbacks that the loader is being torn down.
+            for (auto &callback : ze_lib::context->teardownCallbacks) {
+                callback.second();
+            }
+            // Clear the registered callbacks now that they have been called.
+            ze_lib::context->teardownCallbacks.clear();
+        }
+
         ze_lib::destruction = true;
     }
     return result;
@@ -398,128 +431,123 @@ zelSetDriverTeardown()
 void ZE_APICALL
 zelSetDelayLoaderContextTeardown()
 {
-    #ifdef DYNAMIC_LOAD_LOADER
+    #ifdef L0_STATIC_LOADER_BUILD
     if (!ze_lib::delayContextDestruction) {
         ze_lib::delayContextDestruction = true;
     }
     #endif
 }
 
-#ifdef DYNAMIC_LOAD_LOADER
-#define ZEL_STABILITY_CHECK_RESULT_SUCCESS 0
-#define ZEL_STABILITY_CHECK_RESULT_DRIVER_GET_NULL 1
-#define ZEL_STABILITY_CHECK_RESULT_DRIVER_GET_FAILED 2
-#define ZEL_STABILITY_CHECK_RESULT_EXCEPTION 3
-
-/**
- * @brief Performs a stability check for the Level Zero loader.
- *
- * This function checks the stability of the Level Zero loader by verifying
- * the presence of the loader module, the validity of the `zeDriverGet` function
- * pointer, and the ability to retrieve driver information. The result of the
- * stability check is communicated through the provided promise.
- *
- * @param stabilityPromise A promise object used to communicate the result of
- *                         the stability check. The promise is set with one of
- *                         the following values:
- *                         - ZEL_STABILITY_CHECK_RESULT_DRIVER_GET_NULL: The
- *                           `zeDriverGet` function pointer is invalid.
- *                         - ZEL_STABILITY_CHECK_RESULT_DRIVER_GET_FAILED: The
- *                           loader failed to retrieve driver information.
- *                         - ZEL_STABILITY_CHECK_RESULT_EXCEPTION: An
- *                           exception occurred during the stability check.
- *                         - ZEL_STABILITY_CHECK_RESULT_SUCCESS: The stability
- *                           check was successful.
- *
- * @note If debug tracing is enabled, debug messages are logged for each failure
- *       scenario.
- * @note If the Loader is completely torn down, this thread is expected to be killed
- *      due to invalid memory access and the stability check will determine a failure.
- *
- * @exception This function catches all exceptions internally and does not throw.
- */
-void stabilityCheck(std::promise<int> stabilityPromise) {
-    try {
-        if (!ze_lib::context->loaderDriverGet) {
-            if (ze_lib::context->debugTraceEnabled) {
-                std::string message = "LoaderDriverGet is a bad pointer. Exiting stability checker thread.";
-                ze_lib::context->debug_trace_message(message, "");
-            }
-            stabilityPromise.set_value(ZEL_STABILITY_CHECK_RESULT_DRIVER_GET_NULL);
-            return;
-        }
-
-        uint32_t driverCount = 0;
-        ze_result_t result = ZE_RESULT_ERROR_UNINITIALIZED;
-        result = ze_lib::context->loaderDriverGet(&driverCount, nullptr);
-        if (result != ZE_RESULT_SUCCESS || driverCount == 0) {
-            if (ze_lib::context->debugTraceEnabled) {
-                std::string message = "Loader stability check failed. Exiting stability checker thread.";
-                ze_lib::context->debug_trace_message(message, "");
-            }
-            stabilityPromise.set_value(ZEL_STABILITY_CHECK_RESULT_DRIVER_GET_FAILED);
-            return;
-        }
-        stabilityPromise.set_value(ZEL_STABILITY_CHECK_RESULT_SUCCESS);
-        return;
-    } catch (...) {
-        stabilityPromise.set_value(ZEL_STABILITY_CHECK_RESULT_EXCEPTION);
-        return;
+/// @brief Registers a teardown callback function to be invoked during loader teardown.
+///
+/// This function allows an application to register a callback that will be called when the loader is being torn down.
+/// The loader provides a callback function pointer to the application, which the application should call to notify
+/// the loader that it is tearing down. The loader will then remove the application's callback from its list of registered callbacks.
+///
+/// @param[in] application_callback
+///     The application's callback function to be called during loader teardown. Must not be nullptr.
+/// @param[out] loader_callback
+///     Pointer to the loader's callback function. The application should call this function to notify the loader of teardown.
+/// @param[out] index
+///     Pointer to a uint32_t that will receive the index assigned to the registered callback.
+///
+/// @return
+///     - ZE_RESULT_SUCCESS: The callback was successfully registered.
+///     - ZE_RESULT_ERROR_INVALID_ARGUMENT: The application_callback parameter is nullptr.
+///     - ZE_RESULT_ERROR_UNINITIALIZED: The loader context is not initialized.
+ze_result_t ZE_APICALL
+zelRegisterTeardownCallback(
+   zel_loader_teardown_callback_t application_callback, // [in] Application's callback function to be called during loader teardown
+   zel_application_teardown_callback_t *loader_callback, // [out] Pointer to the loader's callback function
+   uint32_t *index // [out] Index assigned to the registered callback
+) {
+    ze_result_t result = ZE_RESULT_SUCCESS;
+    if (nullptr == application_callback) {
+        return ZE_RESULT_ERROR_INVALID_ARGUMENT;
     }
+    if (!ze_lib::context) {
+        return ZE_RESULT_ERROR_UNINITIALIZED;
+    }
+    {
+        std::lock_guard<std::mutex> lock(ze_lib::context->teardownCallbacksMutex);
+        // Assign the loader's callback function to the application callback such that the application can notify the loader
+        // that it is tearing down. The loader will then remove the application's callback from the list of callbacks.
+        *loader_callback = ze_lib::applicationTeardownCallback;
+        // Increment the teardown callback count and assign the index to the application callback.
+        ze_lib::context->teardownCallbacksCount.fetch_add(1);
+        *index = ze_lib::context->teardownCallbacksCount.load();
+        ze_lib::context->teardownCallbacks.insert(std::pair<uint32_t, zel_loader_teardown_callback_t>(*index, application_callback));
+        if (ze_lib::context->debugTraceEnabled) {
+            std::string message = "Registered teardown callback with index: " + std::to_string(*index);
+            ze_lib::context->debug_trace_message(message, "");
+        }
+    }
+    return result;
 }
-#endif
 
-/**
- * @brief Checks if the loader is in the process of tearing down.
- *
- * This function determines whether the loader is in a teardown state by
- * checking the destruction flag or the context pointer. If the loader is
- * dynamically loaded thru the static loader code path, then it performs
- * an additional stability check using a separate thread that could be killed.
- *
- * @return true if the loader is in teardown based on the stack variablrs
- *         or the stability check fails; false otherwise.
- *
- * @note If the macro DYNAMIC_LOAD_LOADER is defined, a stability checker
- *       thread is launched to perform additional checks. Any exceptions
- *       or errors during this process are logged if debug tracing is enabled.
- */
+/// @brief Checks if the Level Zero loader is currently in the teardown state.
+///
+/// This function determines whether the loader is in the process of being destroyed or is otherwise
+/// unavailable for further API calls. It performs several checks, including:
+/// - Whether the loader's destruction flag is set or the context is null.
+/// - On Windows with dynamic loading, it checks for loader teardown notifications,
+///   registration status, and the stability of the loader by attempting to call `loaderDriverGet`.
+/// - If any of these checks indicate the loader is in teardown or unstable, the function returns true.
+///
+/// @return true if the loader is in teardown or unstable; false otherwise.
 bool ZE_APICALL
 zelCheckIsLoaderInTearDown() {
     if (ze_lib::destruction || ze_lib::context == nullptr) {
         return true;
     }
-    #ifdef DYNAMIC_LOAD_LOADER
-    std::promise<int> stabilityPromise;
-    std::future<int> resultFuture = stabilityPromise.get_future();
-    int result = -1;
-    try {
-        // Launch the stability checker thread
-        std::thread stabilityThread(stabilityCheck, std::move(stabilityPromise));
-        result = resultFuture.get(); // Blocks until the result is available
+    #if defined(L0_STATIC_LOADER_BUILD) && defined(_WIN32)
+    static bool loaderIsStable = true;
+    if (!loaderIsStable) {
         if (ze_lib::context->debugTraceEnabled) {
-            std::string message = "Stability checker thread completed with result: " + std::to_string(result);
-            ze_lib::context->debug_trace_message(message, "");
-        }
-        stabilityThread.join();
-    } catch (const std::exception& e) {
-        if (ze_lib::context->debugTraceEnabled) {
-            std::string message = "Exception caught in parent thread: " + std::string(e.what());
-            ze_lib::context->debug_trace_message(message, "");
-        }
-    } catch (...) {
-        if (ze_lib::context->debugTraceEnabled) {
-            std::string message = "Unknown exception caught in parent thread.";
-            ze_lib::context->debug_trace_message(message, "");
-        }
-    }
-    if (result != ZEL_STABILITY_CHECK_RESULT_SUCCESS) {
-        if (ze_lib::context->debugTraceEnabled) {
-            std::string message = "Loader stability check failed with result: " + std::to_string(result);
+            std::string message = "Loader Teardown check failed before, exiting.";
             ze_lib::context->debug_trace_message(message, "");
         }
         return true;
     }
+    if (ze_lib::loaderTeardownCallbackReceived) {
+        if (ze_lib::context->debugTraceEnabled) {
+            std::string message = "Loader Teardown Notification Received, loader in teardown state.";
+            ze_lib::context->debug_trace_message(message, "");
+        }
+        loaderIsStable = false;
+        return true;
+    }
+    if (!ze_lib::loaderTeardownRegistrationEnabled) {
+        try {
+            if (!ze_lib::context->loaderDriverGet) {
+                if (ze_lib::context->debugTraceEnabled) {
+                    std::string message = "LoaderDriverGet is a bad pointer. Exiting stability checker.";
+                    ze_lib::context->debug_trace_message(message, "");
+                }
+                loaderIsStable = false;
+                return true;
+            }
+
+            uint32_t driverCount = 0;
+            ze_result_t result = ZE_RESULT_ERROR_UNINITIALIZED;
+            result = ze_lib::context->loaderDriverGet(&driverCount, nullptr);
+            if (result != ZE_RESULT_SUCCESS || driverCount == 0) {
+                if (ze_lib::context->debugTraceEnabled) {
+                    std::string message = "Loader stability check failed. Exiting stability checker.";
+                    ze_lib::context->debug_trace_message(message, "");
+                }
+                loaderIsStable = false;
+                return true;
+            }
+        } catch (...) {
+            if (ze_lib::context->debugTraceEnabled) {
+                std::string message = "Loader stability check failed. Exception occurred.";
+                ze_lib::context->debug_trace_message(message, "");
+            }
+            loaderIsStable = false;
+            return true;
+        }
+    }
     #endif
     return false;
 }
@@ -527,7 +555,7 @@ zelCheckIsLoaderInTearDown() {
 void ZE_APICALL
 zelLoaderContextTeardown()
 {
-    #ifdef DYNAMIC_LOAD_LOADER
+    #ifdef L0_STATIC_LOADER_BUILD
     if (ze_lib::delayContextDestruction && ze_lib::context) {
         delete ze_lib::context;
         ze_lib::context = nullptr;
@@ -538,7 +566,7 @@ zelLoaderContextTeardown()
 ze_result_t ZE_APICALL
 zelEnableTracingLayer()
 {
-    #ifdef DYNAMIC_LOAD_LOADER
+    #ifdef L0_STATIC_LOADER_BUILD
     if(nullptr == ze_lib::context->loader)
         return ZE_RESULT_ERROR_UNINITIALIZED;
     typedef ze_result_t (ZE_APICALL *zelEnableTracingLayerInternal_t)();
@@ -551,6 +579,9 @@ zelEnableTracingLayer()
     }
     if (ze_lib::context->tracingLayerEnableCounter.fetch_add(1) == 0) {
         ze_lib::context->zeDdiTable.exchange(ze_lib::context->pTracingZeDdiTable);
+        if (ze_lib::context->pTracingZerDdiTable != nullptr) {
+            ze_lib::context->zerDdiTable.exchange(ze_lib::context->pTracingZerDdiTable);
+        }
     }
     #endif
     return ZE_RESULT_SUCCESS;
@@ -559,7 +590,7 @@ zelEnableTracingLayer()
 ze_result_t ZE_APICALL
 zelDisableTracingLayer()
 {
-    #ifdef DYNAMIC_LOAD_LOADER
+    #ifdef L0_STATIC_LOADER_BUILD
     if(nullptr == ze_lib::context->loader)
         return ZE_RESULT_ERROR_UNINITIALIZED;
     typedef ze_result_t (ZE_APICALL *zelDisableTracingLayerInternal_t)();
@@ -572,6 +603,9 @@ zelDisableTracingLayer()
     }
     if (ze_lib::context->tracingLayerEnableCounter.fetch_sub(1) <= 1) {
         ze_lib::context->zeDdiTable.exchange(&ze_lib::context->initialzeDdiTable);
+        if (ze_lib::context->pTracingZerDdiTable != nullptr) {
+            ze_lib::context->zerDdiTable.exchange(&ze_lib::context->initialzerDdiTable);
+        }
     }
     #endif
     return ZE_RESULT_SUCCESS;
